class RichText:
    def __init__(self, data=None):
        if data is not None:
            if isinstance(data, dict):
                self.data = data
                self.is_link = data['href'] is not None
                self.href = data['href']
                self.is_bold = data['annotations']['bold']
                self.is_italic = data['annotations']['italic']
                self.is_underlined = data['annotations']['underline']
                self.is_strikethrough = data['annotations']['strikethrough']
                self.is_code = data['annotations']['code']
                self.color = data['annotations']['color']
                self.text = data['plain_text']
            elif isinstance(data, str):
                self.text = data
                self.is_link = False
                self.href = None
                self.is_bold = False
                self.is_italic = False
                self.is_underlined = False
                self.is_strikethrough = False
                self.is_code = False
                self.color = 'default'

    def toggle_bold(self):
        self.is_bold = not self.is_bold
        return self
    def toggle_italic(self):
        self.is_italic = not self.is_italic
        return self
    def toggle_underlined(self):
        self.is_underlined = not self.is_underlined
        return self
    def toggle_striketrough(self):
        self.is_strikethrough = not self.is_strikethrough
        return self

    def set_link(self, href):
        self.is_link = True
        self.href = href
        return self


    def get_plaintext(self):
        return self.text

    def set_text(self, text):
        self.text = text
        return self

    def get_json(self):
        data =  {
            'type': 'text',
            'plain_text': self.text,
            'annotations': {
                'bold': self.is_bold,
                'code': self.is_code,
                'color': self.color,
                'italic': self.is_italic,
                'strikethrough': self.is_strikethrough,
                'underline': self.is_underlined,
            },
            'text': {
                'content': self.text,
            }
        }
        if self.href is not None:
            data['href'] = self.href
            data['text']['link'] = {
                'url': self.href
            }

        return data



import requests
from .page import Page
from .blocks import Block
from .richtext import RichText

class Notion:
    key = None
    api_version = '2021-08-16'

    def __init__(self, key):
        Notion.key = key


    def get_basic_headers(self):
        return {
            'Notion-Version': Notion.api_version,
            'Authorization': 'Bearer ' + Notion.key
        }


    def req_get(self, url, params = {}, headers = {}):
        headers.update(self.get_basic_headers())
        response = requests.get(url, json = params, headers = headers)
        return response.json()

    def req_post(self, url, params = {}, headers = {}):
        headers.update(self.get_basic_headers())
        response = requests.post(url, json = params, headers = headers)
        return response.json()

    def req_put(self, url, params = {}, headers = {}):
        headers.update(self.get_basic_headers())
        response = requests.put(url, json = params, headers = headers)
        return response.json()

    def req_delete(self, url, params = {}, headers = {}):
        headers.update(self.get_basic_headers())
        response = requests.delete(url, json = params, headers = headers)
        return response.json()

    def req_patch(self, url, params = {}, headers = {}):
        headers.update(self.get_basic_headers())
        response = requests.patch(url, json = params, headers = headers)
        return response.json()


    def get_page(self, page_id, lazy = False):
        page = Page(page_id, lazy = lazy, notion_obj = self)
        return page

    
    def db_query(self, db_id, filter = {}, sorts = []):
        data = {}
        if len(filter) != 0:
            data['filter'] = filter
        if len(sorts) != 0:
            data['sorts']

        response = self.req_post(f'https://api.notion.com/v1/databases/{db_id}/query', data)
        result = []
        for page in response['results']:
            result.append(Page(data = page, notion_obj = self))
        return result

    
    def create_rich_text(self, text):
        return RichText(text)

    def create_paragraph(self, rt = None):
        b = Block()
        b.type = 'paragraph'
        if rt is not None:
            if isinstance(rt, list):
                b.rich_text_objects += rt
            else:
                b.append_richtext(rt)
        return b
        


from .properties import Prop
from .blocks import Block
import json

class Page:
    def __init__(self, id = -1, lazy = False, notion_obj = None, data = None):
        self.notion_obj = None # Instance of wrapper. Used to do something with page using requests to Notion
        self.data = {} # page's data
        self.id = None # page's id
        self.content = [] # children blocks of the page
        
        if notion_obj is None:
            raise RuntimeError('Page instance can be created only using Notion object')
        else:
            self.notion_obj = notion_obj
            if data is not None:
                self.data = data
                self.id = data['id']
                self.content = []
            else:
                self.id = id
                self.data = { 
                    'id': id,
                    'raw_content': {
                        'results': []
                    },
                    'plaintext': []
                }
                self.content = []
                if not lazy:
                    self.load_data().load_content()


    def __str__(self):
        return self.get_json()
    def __repr__(self):
        return self.get_json()

    def get_json(self):
        return json.dumps(self.data, indent=4)


    def get_data(self):
        return self.data
    def get_title(self):
        return self.data['properties']['title']['plain_text']
    def get_cover(self):
        return self.data['cover']['external']['url']
    def get_created_time(self):
        return self.data['created_time']

    def get_prop(self, prop):
        return Prop(self.data['properties'][prop])
    def get_prop_value(self, prop):
        return self.get_prop(prop).get_value()
        

    def append_block(self, block):
        data = { 'children': [block.get_json()] }
        self.notion_obj.req_patch(f'https://api.notion.com/v1/blocks/{self.id}/children', data)
        self.data['raw_content']['results'].append(block.get_json())
        self.data['plaintext'].append(block.get_plaintext())
        self.content.append(block)
        return self


    def load_data(self):
        self.data = self.notion_obj.req_get(f'https://api.notion.com/v1/pages/{self.id}')
        return self

    def load_content(self):
        self.data['has_children'] = self.notion_obj.req_get(f'https://api.notion.com/v1/blocks/{self.id}')['has_children']
        if self.data['has_children']:
            self.data['raw_content'] = self.notion_obj.req_get(f'https://api.notion.com/v1/blocks/{self.id}/children')
            self.content = []
            self.data['plaintext'] = []
            for block in self.data['raw_content']['results']:
                b = Block(block)
                self.content.append(b)
                self.data['plaintext'].append(b.get_plaintext())
        return self

    # state-checkers
    def is_data_loaded(self):
        return 'properties' in self.data


from .richtext import RichText
from pprint import pprint

class Block:
    def __init__(self, data = None):
        if data is not None:
            self.data = data
            self.type = data['type']
            self.has_children = data['has_children']
            self.archived = data['archived']
            self.id = data['id']

            self.parse_richtext_list()
        else:
            self.rich_text_objects = []
            self.has_children = False
            self.archived = False

    def parse_richtext_list(self):
        text_list = self.data[self.type]['text']
        self.rich_text_objects = []
        for rt_obj in text_list:
            richtextobj = RichText(rt_obj)
            self.rich_text_objects.append(richtextobj)

    def append_richtext(self, rt):
        if isinstance(rt, list):
            self.rich_text_objects.extend(rt)
        else:
            self.rich_text_objects.append(rt)
        return self

    def get_plaintext(self):
        result = ''
        for rto in self.rich_text_objects:
            result += rto.get_plaintext()
        return result

    def get_json(self):
        rt_list = []
        for rt in self.rich_text_objects:
            rt_list.append(rt.get_json())
        return {
            'object': 'block',
            'type': self.type,
            'archived': False,
            'has_children': self.has_children,
            self.type: {
                'text': rt_list
            }
        }


import json

# TODO add People, File & Formula filters
class CompoundFilter:
    def __init__(self, filter1 = None, filter2 = None, op = None):
        self.update(filter1, filter2, op)

    def __str__(self):
        return self.get_json()
    def __repr__(self):
        return self.get_json()

    def get_json(self):
        return json.dumps(self.filter_obj, indent=4)

    def update(self, filter1 = None, filter2 = None, op = "or"):
        self.filter_obj = CompoundFilter.create(filter1, filter2, op)
        return self.filter_obj
    
    def get(self):
        return self.filter_obj

    @staticmethod
    def create(filter1 = None, filter2 = None, op = "or"):
        if not filter1 is None and not filter2 is None:
            filter_obj = {
                op: [
                    filter1, filter2
                ]
            }
            return filter_obj
        else:
            raise RuntimeError("You must provide both filters to create CompoundFilter")

class TextFilter:
    @staticmethod  
    def equals(prop, val):
        return {
            "property": prop,
            "text": {
                "equals": val
            }
        }
    @staticmethod
    def not_equals(prop, val):
        return {
            "property": prop,
            "text": {
                "does_not_equal": val
            }
        }

    @staticmethod
    def contains(prop, val):
        return {
            "property": prop,
            "text": {
                "contains": val
            }
        }
    @staticmethod
    def not_contains(prop, val):
        return {
            "property": prop,
            "text": {
                "does_not_contain": val
            }
        }

    @staticmethod
    def starts_with(prop, val):
        return {
            "property": prop,
            "text": {
                "starts_with": val
            }
        }
    @staticmethod
    def ends_with(prop, val):
        return {
            "property": prop,
            "text": {
                "ends_with": val
            }
        }

    @staticmethod
    def is_empty(prop, val):
        filter_object = {
            "property": prop
        }
        if val:
            filter_object['text']['is_empty'] = True
        else:
            filter_object['text']['is_not_empty'] = True
        return filter_object




class NumberFilter:
    @staticmethod  
    def equals(prop, val):
        return {
            "property": prop,
            "number": {
                "equals": val
            }
        }
    @staticmethod  
    def not_equals(prop, val):
        return {
            "property": prop,
            "number": {
                "does_not_equal": val
            }
        }

    @staticmethod  
    def greater(prop, val):
        return {
            "property": prop,
            "number": {
                "greater_than": val
            }
        }
    @staticmethod  
    def less(prop, val):
        return {
            "property": prop,
            "number": {
                "less_than": val
            }
        }

    # TODO: maybe I should add filters for greater_than_or_equal_to & less_than_or_equal_to

    @staticmethod
    def is_empty(prop, val):
        filter_object = {
            "property": prop
        }
        if val:
            filter_object['number']['is_empty'] = True
        else:
            filter_object['number']['is_not_empty'] = True
        return filter_object



class CheckboxFilter:
    @staticmethod  
    def checked(prop, val):
        return {
            "property": prop,
            "boolean": {
                "equals": val
            }
        }




class SelectFilter:
    @staticmethod  
    def equals(prop, val):
        return {
            "property": prop,
            "select": {
                "equals": val
            }
        }
    @staticmethod  
    def not_equals(prop, val):
        return {
            "property": prop,
            "select": {
                "equals": val
            }
        }

    @staticmethod
    def is_empty(prop, val):
        filter_object = {
            "property": prop
        }
        if val:
            filter_object['select']['is_empty'] = True
        else:
            filter_object['select']['is_not_empty'] = True
        return filter_object




class MultiSelectFilter:
    @staticmethod  
    def contains(prop, val):
        return {
            "property": prop,
            "multi_select": {
                "contains": val
            }
        }
    @staticmethod  
    def not_contains(prop, val):
        return {
            "property": prop,
            "multi_select": {
                "does_not_contain": val
            }
        }

    @staticmethod
    def is_empty(prop, val):
        filter_object = {
            "property": prop
        }
        if val:
            filter_object['multi_select']['is_empty'] = True
        else:
            filter_object['multi_select']['is_not_empty'] = True
        return filter_object




class RelationFilter:
    @staticmethod  
    def contains(prop, val):
        return {
            "property": prop,
            "relation": {
                "contains": val
            }
        }
    @staticmethod  
    def not_contains(prop, val):
        return {
            "property": prop,
            "relation": {
                "does_not_contain": val
            }
        }

    @staticmethod
    def is_empty(prop, val):
        filter_object = {
            "property": prop
        }
        if val:
            filter_object['relation']['is_empty'] = True
        else:
            filter_object['relation']['is_not_empty'] = True
        return filter_object




class DateFilter:
    @staticmethod  
    def equals(prop, val, type="date"):
        return {
            "property": prop,
            type: {
                "equals": val
            }
        }

    @staticmethod  
    def before(prop, val, type="date"):
        return {
            "property": prop,
            type: {
                "before": val
            }
        }
    @staticmethod  
    def after(prop, val, type="date"):
        return {
            "property": prop,
            type: {
                "after": val
            }
        }

    @staticmethod  
    def past_week(prop, val, type="date"):
        return {
            "property": prop,
            type: {
                "past_week": {}
            }
        }
    def next_week(prop, val, type="date"):
        return {
            "property": prop,
            type: {
                "next_week": {}
            }
        }

    @staticmethod  
    def past_month(prop, val, type="date"):
        return {
            "property": prop,
            type: {
                "past_month": {}
            }
        }
    def next_month(prop, val, type="date"):
        return {
            "property": prop,
            type: {
                "next_month": {}
            }
        }

    @staticmethod  
    def past_year(prop, val, type="date"):
        return {
            "property": prop,
            type: {
                "past_year": {}
            }
        }
    def next_year(prop, val, type="date"):
        return {
            "property": prop,
            type: {
                "next_year": {}
            }
        }

    @staticmethod
    def is_empty(prop, val, type="date"):
        filter_object = {
            "property": prop
        }
        if val:
            filter_object[type]['is_empty'] = True
        else:
            filter_object[type]['is_not_empty'] = True
        return filter_object


import json

class Sort:
    @staticmethod
    def asc(prop):
        return [
            {
                "property": prop,
                "direction": "ascending"
            }
        ]
    @staticmethod
    def desc(prop):
        return [
            {
                "property": prop,
                "direction": "descending"
            }
        ]

class CompoundSort:
    def __init__(self, sorts):
        self.update(sorts)

    def __str__(self):
        return self.get_json()
    def __repr__(self):
        return self.get_json()

    def get_json(self):
        return json.dumps(self.sorts_list, indent=4)

    def update(self, sorts):
        self.sorts_list = CompoundSort.create(sorts)
        return self.sorts_list
    
    def get(self):
        return self.sorts_list

    @staticmethod
    def create(sorts):
        sorts_list = []
        for sort in sorts:
            sorts_list.append(sort[0])
        return sorts_list


import json

class Prop:
    def __init__(self, data):
        self.data = data
        self.type = data['type']

    def __str__(self):
        return self.get_json()
    def __repr__(self):
        return self.get_json()

    def get_json(self):
        return json.dumps(self.data, indent=4)
    
    def get_value(self):
        if self.type == 'title':
            return self.data['title']['plain_text']
        else:
            return self.data[type]


from setuptools import setup
from pynotio.version import version

with open('README.md', 'r') as fh:
    long_description = fh.read()

requirements = ['requests<=2.27.1']


setup(name='pynotio',
      version=version,
      description='Python wrapper around official Notion API for easy life',
      long_description=long_description,
      packages=['pynotio'],
      author='Ruslan Abkadirov',
      author_email='rusabk@yandex.ru',
      url='https://github.com/Ruslanabk/PyNotio',
      zip_safe=False,
      install_requires=requirements,
      classifiers=[
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
      ],
      python_requires='>=3.6'
)


"""
Pynotio is designed to be a really simple Python wrapper around the official Notion API.

Ruslan Abkadirov [https://blog.ruslanabk.ru/]
rusabk@yandex.ru
# License: Apache License 2.0
"""

from .version import version

__author__ = 'Ruslan Abkadirov'
__version__ = version